"""Django Rubble app configuration."""

from django.apps import AppConfig


class UtilsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_rubble"
    verbose_name = "Django Rubble"
